# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from PyQt4.QtGui import QSplitter

from ....model import adapt, Adapter
from ....settings import ISettings
from ....ui import ISplitter

from ..utils import as_QWidget, from_QWidget
from ..widget_container_adapter import WidgetContainerAdapter


@adapt(QSplitter, to=ISplitter)
class QSplitterISplitterAdapter(WidgetContainerAdapter):
    """ An adapter to implement ISplitter for a QSplitter. """

    @ISplitter.views.getter
    def views(self):
        """ Invoked to get the views. """

        splitter = self.adaptee

        return tuple(from_QWidget(splitter.widget(idx))
                for idx in range(splitter.count()))

    @views.setter
    def views(self, value):
        """ Invoked to set the views. """

        splitter = self.adaptee

        # FIXME: Remove any existing widgets.
        for item in value:
            splitter.addWidget(as_QWidget(item))


@adapt(QSplitter, to=ISettings)
class QSplitterISettingsAdapter(Adapter):
    """ An adapter to implement ISettings for a QSplitter. """

    @ISettings.id.getter
    def id(self):
        """ Invoked to get the identifier. """

        return self.adaptee.objectName()

    def restore(self, settings_manager):
        """ Restore the settings obtained from a settings manager.

        :param settings_manager:
            is the settings manager.
        """

        value = settings_manager.read_value('state')
        if value is not None:
            self.adaptee.restoreState(value)

    def save(self, settings_manager):
        """ Save the settings to a settings manager.

        :param settings_manager:
            is the settings manager.
        """

        settings_manager.write_value('state', self.adaptee.saveState())
